@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 ##
 # Licensed to the Apache Software Foundation (ASF) under one
@@ -17,66 +17,57 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import, division, print_function
-
+import abc
 import collections
 import io
 import logging
 import os
-import sys
 import threading
 import traceback
+from typing import cast
 
+import avro.errors
 import avro.io
-from avro import ipc, protocol, schema
+import avro.ipc
+import avro.protocol
+import avro.schema
 
 __all__ = ["TetherTask", "TaskType", "inputProtocol", "outputProtocol", "HTTPRequestor"]
 
 # create protocol objects for the input and output protocols
 # The build process should copy InputProtocol.avpr and OutputProtocol.avpr
 # into the same directory as this module
-inputProtocol = None
-outputProtocol = None
 
 TaskType = None
-if (inputProtocol is None):
-    pfile = os.path.split(__file__)[0] + os.sep + "InputProtocol.avpr"
+pfile = os.path.split(__file__)[0] + os.sep + "InputProtocol.avpr"
+with open(pfile) as hf:
+    prototxt = hf.read()
 
-    if not(os.path.exists(pfile)):
-        raise Exception("Could not locate the InputProtocol: {0} does not exist".format(pfile))
+inputProtocol = avro.protocol.parse(prototxt)
 
-    with open(pfile, 'r') as hf:
-        prototxt = hf.read()
+# use a named tuple to represent the tasktype enumeration
+assert inputProtocol.types_dict is not None
+taskschema = cast(avro.schema.EnumSchema, inputProtocol.types_dict["TaskType"])
+# Mypy cannot statically type check a dynamically constructed named tuple.
+# Since InputProtocol.avpr is hard-coded here, we can hard-code the symbols.
+_ttype = collections.namedtuple("_ttype", ("MAP", "REDUCE"))
+TaskType = _ttype(*taskschema.symbols)
 
-    inputProtocol = protocol.parse(prototxt)
+pfile = os.path.split(__file__)[0] + os.sep + "OutputProtocol.avpr"
 
-    # use a named tuple to represent the tasktype enumeration
-    taskschema = inputProtocol.types_dict["TaskType"]
-    # Mypy cannot statically type check a dynamically constructed named tuple.
-    # Since InputProtocol.avpr is hard-coded here, we can hard-code the symbols.
-    _ttype = collections.namedtuple("_tasktype", ("MAP", "REDUCE"))
-    TaskType = _ttype(*taskschema.symbols)
+with open(pfile) as hf:
+    prototxt = hf.read()
 
-if (outputProtocol is None):
-    pfile = os.path.split(__file__)[0] + os.sep + "OutputProtocol.avpr"
-
-    if not(os.path.exists(pfile)):
-        raise Exception("Could not locate the OutputProtocol: {0} does not exist".format(pfile))
-
-    with open(pfile, 'r') as hf:
-        prototxt = hf.read()
-
-    outputProtocol = protocol.parse(prototxt)
+outputProtocol = avro.protocol.parse(prototxt)
 
 
-class Collector(object):
+class Collector:
     """
     Collector for map and reduce output values
     """
 
-    def __init__(self, scheme=None, outputClient=None):
+    def __init__(self, scheme, outputClient):
         """
-
         Parameters
         ---------------------------------------------
         scheme - The scheme for the datums to output - can be a json string
@@ -84,11 +75,8 @@ class Collector(object):
         outputClient - The output client used to send messages to the parent
         """
 
-        if not(isinstance(scheme, schema.Schema)):
-            scheme = schema.parse(scheme)
-
-        if (outputClient is None):
-            raise ValueError("output client can't be none.")
+        if not isinstance(scheme, avro.schema.Schema):
+            scheme = avro.schema.parse(scheme)
 
         self.scheme = scheme
 
@@ -128,13 +116,13 @@ def keys_are_equal(rec1, rec2, fkeys):
     """
 
     for f in fkeys:
-        if not(rec1[f] == rec2[f]):
+        if not (rec1[f] == rec2[f]):
             return False
 
     return True
 
 
-class HTTPRequestor(object):
+class HTTPRequestor:
     """
     This is a small requestor subclass I created for the HTTP protocol.
     Since the HTTP protocol isn't persistent, we need to instantiate
@@ -159,12 +147,12 @@ class HTTPRequestor(object):
         self.protocol = protocol
 
     def request(self, *args, **param):
-        transciever = ipc.HTTPTransceiver(self.server, self.port)
-        requestor = ipc.Requestor(self.protocol, transciever)
+        transciever = avro.ipc.HTTPTransceiver(self.server, self.port)
+        requestor = avro.ipc.Requestor(self.protocol, transciever)
         return requestor.request(*args, **param)
 
 
-class TetherTask(object):
+class TetherTask(abc.ABC):
     """
     Base class for python tether mapreduce programs.
 
@@ -179,7 +167,7 @@ class TetherTask(object):
     away but wait for space to free up)
     """
 
-    def __init__(self, inschema=None, midschema=None, outschema=None):
+    def __init__(self, inschema, midschema, outschema):
         """
 
         Parameters
@@ -203,21 +191,11 @@ class TetherTask(object):
         the differences (see https://avro.apache.org/docs/current/spec.html#Schema+Resolution))
 
         """
-
-        if (inschema is None):
-            raise ValueError("inschema can't be None")
-
-        if (midschema is None):
-            raise ValueError("midschema can't be None")
-
-        if (outschema is None):
-            raise ValueError("outschema can't be None")
-
         # make sure we can parse the schemas
         # Should we call fail if we can't parse the schemas?
-        self.inschema = schema.parse(inschema)
-        self.midschema = schema.parse(midschema)
-        self.outschema = schema.parse(outschema)
+        self.inschema = avro.schema.parse(inschema)
+        self.midschema = avro.schema.parse(midschema)
+        self.outschema = avro.schema.parse(outschema)
 
         # declare various variables
         self.clienTransciever = None
@@ -266,35 +244,23 @@ class TetherTask(object):
         # The port the parent process is listening on is set in the environment
         # variable AVRO_TETHER_OUTPUT_PORT
         # open output client, connecting to parent
+        clientPort = int(clientPort or os.getenv("AVRO_TETHER_OUTPUT_PORT", 0))
+        if clientPort == 0:
+            raise avro.errors.UsageError("AVRO_TETHER_OUTPUT_PORT env var is not set")
 
-        if (clientPort is None):
-            clientPortString = os.getenv("AVRO_TETHER_OUTPUT_PORT")
-            if (clientPortString is None):
-                raise Exception("AVRO_TETHER_OUTPUT_PORT env var is not set")
+        self.log.info("TetherTask.open: Opening connection to parent server on port=%d", clientPort)
 
-            clientPort = int(clientPortString)
-
-        self.log.info("TetherTask.open: Opening connection to parent server on port={0}".format(clientPort))
-
-        # We use the HTTP protocol although we hope to shortly have
-        # support for SocketServer,
-        usehttp = True
-
-        if(usehttp):
-            # self.outputClient =  ipc.Requestor(outputProtocol, self.clientTransceiver)
-            # since HTTP is stateless, a new transciever
-            # is created and closed for each request. We therefore set clientTransciever to None
-            # We still declare clientTransciever because for other (state) protocols we will need
-            # it and we want to check when we get the message fail whether the transciever
-            # needs to be closed.
-            # self.clientTranciever=None
-            self.outputClient = HTTPRequestor("127.0.0.1", clientPort, outputProtocol)
-
-        else:
-            raise NotImplementedError("Only http protocol is currently supported")
+        # self.outputClient =  avro.ipc.Requestor(outputProtocol, self.clientTransceiver)
+        # since HTTP is stateless, a new transciever
+        # is created and closed for each request. We therefore set clientTransciever to None
+        # We still declare clientTransciever because for other (state) protocols we will need
+        # it and we want to check when we get the message fail whether the transciever
+        # needs to be closed.
+        # self.clientTranciever=None
+        self.outputClient = HTTPRequestor("127.0.0.1", clientPort, outputProtocol)
 
         try:
-            self.outputClient.request('configure', {"port": inputport})
+            self.outputClient.request("configure", {"port": inputport})
         except Exception:
             estr = traceback.format_exc()
             self.fail(estr)
@@ -318,20 +284,20 @@ class TetherTask(object):
         self.taskType = taskType
 
         try:
-            inSchema = schema.parse(inSchemaText)
-            outSchema = schema.parse(outSchemaText)
+            inSchema = avro.schema.parse(inSchemaText)
+            outSchema = avro.schema.parse(outSchemaText)
 
-            if (taskType == TaskType.MAP):
+            if taskType == TaskType.MAP:
                 self.inReader = avro.io.DatumReader(writers_schema=inSchema, readers_schema=self.inschema)
                 self.midCollector = Collector(outSchemaText, self.outputClient)
 
-            elif(taskType == TaskType.REDUCE):
+            elif taskType == TaskType.REDUCE:
                 self.midReader = avro.io.DatumReader(writers_schema=inSchema, readers_schema=self.midschema)
                 # this.outCollector = new Collector<OUT>(outSchema);
                 self.outCollector = Collector(outSchemaText, self.outputClient)
 
                 # determine which fields in the input record are they keys for the reducer
-                self._red_fkeys = [f.name for f in self.midschema.fields if not(f.order == 'ignore')]
+                self._red_fkeys = [f.name for f in self.midschema.fields if not (f.order == "ignore")]
 
         except Exception as e:
 
@@ -348,7 +314,7 @@ class TetherTask(object):
         self._partitions = npartitions
 
     def input(self, data, count):
-        """ Recieve input from the server
+        """Recieve input from the server
 
         Parameters
         ------------------------------------------------------
@@ -362,20 +328,20 @@ class TetherTask(object):
             decoder = avro.io.BinaryDecoder(bdata)
 
             for i in range(count):
-                if (self.taskType == TaskType.MAP):
+                if self.taskType == TaskType.MAP:
                     inRecord = self.inReader.read(decoder)
 
                     # Do we need to pass midCollector if its declared as an instance variable
                     self.map(inRecord, self.midCollector)
 
-                elif (self.taskType == TaskType.REDUCE):
+                elif self.taskType == TaskType.REDUCE:
 
                     # store the previous record
                     prev = self.midRecord
 
                     # read the new record
                     self.midRecord = self.midReader.read(decoder)
-                    if (prev is not None and not(keys_are_equal(self.midRecord, prev, self._red_fkeys))):
+                    if prev is not None and not (keys_are_equal(self.midRecord, prev, self._red_fkeys)):
                         # since the key has changed we need to finalize the processing
                         # for this group of key,value pairs
                         self.reduceFlush(prev, self.outCollector)
@@ -383,23 +349,24 @@ class TetherTask(object):
 
         except Exception as e:
             estr = traceback.format_exc()
-            self.log.warning("failing: " + estr)
+            self.log.warning("failing: %s", estr)
             self.fail(estr)
 
     def complete(self):
         """
         Process the complete request
         """
-        if ((self.taskType == TaskType.REDUCE) and not(self.midRecord is None)):
+        if (self.taskType == TaskType.REDUCE) and not (self.midRecord is None):
             try:
                 self.reduceFlush(self.midRecord, self.outCollector)
             except Exception as e:
                 estr = traceback.format_exc()
-                self.log.warning("failing: " + estr)
+                self.log.warning("failing: %s", estr)
                 self.fail(estr)
 
         self.outputClient.request("complete", dict())
 
+    @abc.abstractmethod
     def map(self, record, collector):
         """Called with input values to generate intermediat values (i.e mapper output).
 
@@ -412,10 +379,9 @@ class TetherTask(object):
         subclass.
         """
 
-        raise NotImplementedError("This is an abstract method which should be overloaded in the subclass")
-
+    @abc.abstractmethod
     def reduce(self, record, collector):
-        """ Called with input values to generate reducer output. Inputs are sorted by the mapper
+        """Called with input values to generate reducer output. Inputs are sorted by the mapper
         key.
 
         The reduce function is invoked once for each value belonging to a given key outputted
@@ -430,8 +396,7 @@ class TetherTask(object):
         subclass.
         """
 
-        raise NotImplementedError("This is an abstract method which should be overloaded in the subclass")
-
+    @abc.abstractmethod
     def reduceFlush(self, record, collector):
         """
         Called with the last intermediate value in each equivalence run.
@@ -442,7 +407,6 @@ class TetherTask(object):
         ------------------------------------------------------------------
         record - the last record on which reduce was invoked.
         """
-        raise NotImplementedError("This is an abstract method which should be overloaded in the subclass")
 
     def status(self, message):
         """
@@ -460,7 +424,7 @@ class TetherTask(object):
         """
         Call to fail the task.
         """
-        self.log.error("TetherTask.fail: failure occured message follows:\n{0}".format(message))
+        self.log.error("TetherTask.fail: failure occured message follows:\n%s", message)
         try:
             message = message.decode()
         except AttributeError:
@@ -475,7 +439,7 @@ class TetherTask(object):
 
     def close(self):
         self.log.info("TetherTask.close: closing")
-        if not(self.clienTransciever is None):
+        if not (self.clienTransciever is None):
             try:
                 self.clienTransciever.close()
 
